@@ -191,13 +191,15 @@ class MiniTest::Spec < MiniTest::Unit::TestCase
     @specs ||= 0
     @specs += 1
 
-    name = "test_%04d_%s" % [ @specs, desc.gsub(/\W+/, '_').downcase ]
+    name = "test_%04d_%s" % [ @specs, desc ]
 
     define_method name, &block
 
     self.children.each do |mod|
       mod.send :undef_method, name if mod.public_method_defined? name
     end
+
+    name
   end
 
   ##
@@ -238,6 +240,14 @@ class MiniTest::Spec < MiniTest::Unit::TestCase
   end
 
   # :stopdoc:
+  def after_setup
+    run_setup_hooks
+  end
+
+  def before_teardown
+    run_teardown_hooks
+  end
+
   class << self
     attr_reader :desc
     alias :specify :it
@@ -257,7 +267,7 @@ module MiniTest::Expectations
   #
   # :method: must_be_empty
 
-  infect_an_assertion :assert_empty, :must_be_empty
+  infect_an_assertion :assert_empty, :must_be_empty, :unary
 
   ##
   # See MiniTest::Assertions#assert_equal
@@ -417,7 +427,7 @@ module MiniTest::Expectations
   #
   # :method: wont_be_empty
 
-  infect_an_assertion :refute_empty, :wont_be_empty
+  infect_an_assertion :refute_empty, :wont_be_empty, :unary
 
   ##
   # See MiniTest::Assertions#refute_equal
